#!/usr/bin/perl -w

# NagiosGrapher.pm - The tools for the NagiosGrapher.
# Copyright (C) 2004, NETWAYS GmbH, Gerd Mueller, Marius Hein
# $Id: NagiosGrapher.pm 1229 2006-05-24 14:37:26Z mhein $

#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# $Id: NagiosGrapher.pm 1229 2006-05-24 14:37:26Z mhein $

package NagiosGrapher;

delete @ENV{qw(IFS CDPATH ENV BASH_ENV)};

use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION $DEBUG $ngraphcfg $LOG_HOOK $LOG_CONFIG);
use Data::Dumper;
use File::Copy;
use File::Basename;
use Carp;
use POSIX;
use Time::HiRes qw(gettimeofday tv_interval);
use Storable qw(retrieve store);
use GD;
use RRDs;
use Time::Local;

require Exporter;

our @ISA       = qw(Exporter);
our @EXPORT    = qw($VERSION $ngraphcfg);
our @EXPOER_OK = qw();
our $VERSION   = '1.6.1-rc3';
our $DEBUG     = 0;
our @colors    = qw(fff200 ff0000 f24ac8 ed7600 c4fd3d 6dc8fe 00ff00 0000ff);
our $ngraphcfg = '/etc/nagios/ngraph.ncfg';
our $xmlfile   = 'index.xml';
our $uniqueInstance = undef;
our $object_cache   = '/var/run/nagiosgrapher/NagiosGrapher.obj';

$LOG_CONFIG = 128;
$LOG_HOOK = 256;

=begin testing

	ok( require NagiosGrapher, 'load library' );

=end testing

=cut

sub new {
	my ($package) = @_;

	my $self = bless {
		rrdhash        => undef,
		loaded_rrdhash => 0,

		ngraphdef        => undef,
		nmgraphdef       => undef,
		loaded_ngraphdef => 0,

		ngraphcfg        => undef,
		loaded_ngraphcfg => 0,

		configuration        => undef,
		loaded_configuration => 0,

		colors       => \@colors,
		colors_count => 0,

		debug => undef,

		time_start => undef,
		time_stop  => undef,

		loaded_cgi                  => 0,
		use_auth                    => 0,
		authorized_for_all_services => undef,

		nagiosversion => 1,

		log_level => 0,
		log_file  => undef,

	}, $package;

	return $self;
}

sub get_hooks {
	my $self = shift;
	my $type = shift;
	my ($key, @hooks);

	if ($type eq 'before_serviceext') {
		$key = 'hooks_serviceext';
	}
	elsif ($type eq 'before_rrdupdate') {
		$key = 'hooks_rrdupdate';
	}
	elsif ($type eq 'before_imagegraph' || $type eq 'before_imagemultigraph') {
		$key = 'hooks_imagegraph';
	}

	if ($key) {
		my $string = $self->get_ngraphercfg_value($key);
		if ($string) {
			$string =~ s/\s//gi;
			my @tmp = split(/,/, $string);
			push @hooks, @tmp;
		}
	}

	return @hooks;
}

sub call_hook {
	my $self = shift;
	my %args = (
		'ARGSRef' => undef,
		'Host' => undef,
		'Service' => undef,

		'Type' => 'before_serviceext',
		@_
	);

	my @hooks = $self->get_hooks($args{'Type'});
	if (scalar @hooks > 0) {
		foreach my $class (@hooks) {
			my  $check = 0;

			$self->print_log('HOOK: try to invoke '. $class)
				if ($self->{log_level} & $LOG_HOOK);

			eval (
				"use $class;"
			);

			if ($self->{log_level} & $LOG_HOOK && $@) {
				$self->print_log('HOOK: error include module: '. $class);
			}
			else {
				$check = 1;
			}


			if ($check == 1) {

				my $instance = undef;

				eval {
					$instance = $class->new();
				};

				$self->print_log('HOOK: instance created: '. ref($instance))
					if (!$@ && $self->{log_level} & $LOG_HOOK);

				eval {

					$instance->init(
						Values => $args{'ARGSRef'},
						Service => $args{'Service'},
						Host => $args{'Host'},
					);

				};

				$self->print_log('HOOK: module initiated')
					if (!$@ && $self->{log_level} & $LOG_HOOK);

				eval {

					if (!$instance->CheckType($args{'Type'}) && $self->{log_level} & $LOG_HOOK) {
						$self->print_log('HOOK: Type \''. $args{'Type'}. '\' does not match!');
						die();
					}

				};

				$self->print_log('HOOK: module type validated')
					if (!$@ && $self->{log_level} & $LOG_HOOK);

				my $re = undef;

				eval {
					$re = $instance->prepare();
				};

				$self->print_log('HOOK: successfully called prepare()')
					if (!$@ && $self->{log_level} & $LOG_HOOK && $re);

				if ($re) {

					eval {
						$instance->commit();
					};

					$self->print_log('HOOK: successfully called commit()')
						if (!$@ && $self->{log_level} & $LOG_HOOK);

					eval {
						$instance->cleanup();
					};

					$self->print_log('HOOK: successfully called cleanup()')
						if (!$@ && $self->{log_level} & $LOG_HOOK);


				}

			}
		}
	}

	return 1;
}

sub set_log_level {
	my $self = shift;

	my ($log_level) = @_;
	$self->{log_level} = $log_level;
}

sub get_log_level {
	my $self = shift;

	return $self->{log_level};
}

sub set_log_file {
	my $self = shift;

	my ($log_file) = @_;
	$self->{log_file} = $log_file;
}

# Write to file and/or to screen
sub print_log {
	my $self = shift;
	my ($out) = @_;
	chomp($out);
	open( LOG, ">>" . $self->{log_file} )
	  || die "Can't open logfile (" . $self->{log_file} . ")!\n";
	my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) =
	  localtime(time);
	printf( LOG "%02d-%02d-%02d %02d:%02d:%02d %s\n",
		$year + 1900,
		$mon + 1, $mday, $hour, $min, $sec, $out
	);

	# ToDo: Fails on CGI environment ...
	#printf(
	#	"%02d-%02d-%02d %02d:%02d:%02d %s\n",
	#	$year + 1900,
	#	$mon + 1, $mday, $hour, $min, $sec, $out
	#);
	close(LOG);
}

sub read_cgi_cfg {
	my $self = shift;

	my $cgifile = $self->get_ngraphercfg_value('cgi_config');
	if ( open( CGI, "<" . $cgifile ) ) {
		while (<CGI>) {
			if (m/^\s*use_authentication\s*=\s*([0-9])/) {
				$self->{use_auth} = $1;
			}
			elsif (m/^\s*authorized_for_all_services\s*=\s*(.*)$/) {
				$self->{authorized_for_all_services} = $1;
			}
		}
		close(CFG);
		$self->{loaded_cgi} = 1;
	}
}

sub get_use_auth {
	my $self = shift;

	$self->read_cgi_cfg() if ( !$self->{loaded_cgi} );
	return ( $self->{use_auth} );
}

sub get_authorized_for_all_services {
	my $self = shift;

	$self->read_cgi_cfg() if ( !$self->{loaded_cgi} );
	return ( $self->{authorized_for_all_services} );
}

sub read_nagios_cfg {
	my $self = shift;

	my @cfgfiles;
	my ($cfg) = @_;

	# @configuration=();
	$self->{configuration} = ();

	if ( open( CFG, "<" . $cfg ) ) {
		while (<CFG>) {
			if (m/^cfg_file/) {

				# Nagios V.1.X
				my ( $dummy, $cfgfile ) = split(/=/);
				push( @cfgfiles, $cfgfile );
				$self->{nagiosversion} = 1;
			}
			elsif (m/^object_cache_file/) {

				# Nagios V.2.X and higher
				my ( $dummy, $cfgfile ) = split(/=/);
				@cfgfiles = ();
				push( @cfgfiles, $cfgfile );
				$self->{nagiosversion} = 2;
				last;
			}
		}
		close(CFG);
	}

	# read configurations
	foreach (@cfgfiles) {
		if ( open( CFG, "<" . $_ ) ) {
			while (<CFG>) {
				push( @{ $self->{configuration} }, $_ );
			}
			close(CFG);
		}
	}
	$self->{loaded_configuration} = 1;
	return;
}

sub parse_nagios_cfg {
	my $self = shift;

	my $rw           = "";
	my $act_item     = "";
	my $act_subvalue = "";
	my ( $type, $item, $line, $searchtype, $subitem, $subvalue ) = @_;
	$searchtype = 'name' if ( !defined($searchtype) );
	return "" if ( $item eq "" );

	if ( !defined($subitem) ) {
		$subvalue = "";
		$subitem  = "";
	}

	my @items = split( /,/, $item );

	# parse all items
	foreach $item (@items) {
		$rw .= "," if ( $rw ne "" );
		foreach ( @{ $self->{configuration} } ) {

			# item defined?
			if ( /define $type {/ .. /\t}/ ) {

				# host_name, service_description, ...
				my $search = $type . '_' . $searchtype;

				# is this the item we are looking for?
				if (m/$search\s*(.*)/) {
					$act_item = $1;
				}
				elsif ( m/$subitem\s*(.*)/ && $subitem ne "" ) {
					$act_subvalue = $1;

					# return the line we want
				}
				elsif ($act_item eq $item
					&& $subvalue eq $act_subvalue
					&& m/\s*$line\s*(.*)/ )
				{
					$rw .= $1;
				}
			}
		}
	}
	return $rw;
}

sub StepFromCache {
	my ( $step, $nagios_config, $minutes );
	my $self = shift;
	my ( $host, $service, $default_step ) = @_;
	$default_step = $self->get_ngraphercfg_value('step')
	  if ( !defined($default_step) );

	if ( !$self->{loaded_configuration} ) {
		$nagios_config = $self->get_ngraphercfg_value('nagios_config');
		$self->read_nagios_cfg($nagios_config);
	}

	if ( $self->{nagiosversion} == 2 ) {
		$minutes =
		  $self->parse_nagios_cfg( 'service', $service, 'normal_check_interval',
			'description', "host_name", $host );
		if ( defined($minutes) && $minutes > 0 ) {
			$step = $minutes * 60;
		}
		else {
			$step = $default_step;
		}
	}
	else {
		$step = $default_step;
	}

	return $step;
}

sub MakeVlines {
	my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday ) =
	  localtime( time() );

	my $self = shift;

	my ( $start, $graphs, $options, $width ) = @_;

	if ( $start eq "--start=-5400" ) {

		# mark hours
		push( @$graphs,
			    "VRULE:"
			  . timelocal( 0, 0, $hour, $mday, $mon, $year )
			  . "#FF0000" );
		( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday ) =
		  localtime( time() - 3600 );
		push( @$graphs,
			    "VRULE:"
			  . timelocal( 0, 0, $hour, $mday, $mon, $year )
			  . "#FF0000" );

	}
	elsif ( $start eq "--start=-129600" ) {

		# mark days
		push( @$graphs,
			"VRULE:" . timelocal( 0, 0, 0, $mday, $mon, $year ) . "#FF0000" );
		( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday ) =
		  localtime( time() - 86400 );
		push( @$graphs,
			"VRULE:" . timelocal( 0, 0, 0, $mday, $mon, $year ) . "#FF0000" );
		push( @$options, '--x-grid=HOUR:1:HOUR:6:HOUR:6:0:%R' )
		  if ( $width < 220 );

	}
	elsif ( $start eq "--start=-907200" ) {

		# mark weeks
		my $monday_seconds = ( $wday - 1 ) * 24 * 60 * 60;
		my ( $sec, $min, $hour, $mday_1, $mon, $year, $wday, $yday ) =
		  localtime( time() - $monday_seconds );
		push( @$graphs,
			"VRULE:" . timelocal( 0, 0, 0, $mday_1, $mon, $year ) . "#FF0000" );

		( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday ) =
		  localtime( time() - 604800 );
		$monday_seconds = ( $wday - 1 ) * 24 * 60 * 60;
		my $monday = ( localtime( time() - $monday_seconds - 604800 ) )[3];
		$mon-- if ( $monday > $mday_1 );
		if ($mon == -1) {
 			$mon = 11;
			$year--;
		}
		push( @$graphs,
			"VRULE:" . timelocal( 0, 0, 0, $monday, $mon, $year ) . "#FF0000" );

	}
	elsif ( $start eq "--start=-4017600" ) {

		# mark month
		push( @$graphs,
			"VRULE:" . timelocal( 0, 0, 0, 1, $mon, $year ) . "#FF0000" );
		( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday ) =
		  localtime( time() - 2419200 );
		push( @$graphs,
			"VRULE:" . timelocal( 0, 0, 0, 1, $mon, $year ) . "#FF0000" );

		# calendarweek workaround
		push( @$options, '--x-grid=DAY:1:WEEK:1:WEEK:1:604800:Week %V' );

	}
	elsif ( $start eq "--start=-48211200" ) {

		# mark year
		push( @$graphs,
			"VRULE:" . timelocal( 0, 0, 0, 1, 0, $year ) . "#FF0000" );
		( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday ) =
		  localtime( time() - 29030400 );
		push( @$graphs,
			"VRULE:" . timelocal( 0, 0, 0, 1, 0, $year ) . "#FF0000" );
	}

	return;
}

sub AuthCheckType {
	my $self = shift;

	my ($users);
	my $rw = 0;
	my ( $user, $type ) = @_;
	if ( $self->get_use_auth eq "1" ) {
		$users = $self->get_ngraphercfg_value($type);
		if ( defined($users) && length($users) > 0 && $users =~ m/$user/i ) {
			$rw = 1;
		}
	}
	else {
		$rw = 1;
	}
	return ($rw);
}

sub AuthCheck {
	my $self = shift;
	my $rw   = 0;
	my ( $cfg, $host, $user ) = @_;

	$self->read_nagios_cfg($cfg) if ( !$self->{loaded_configuration} );

	if ( $self->{nagiosversion} == 2 ) {

		# Nagios Version 2.? Perform test
		$rw = 1
		  if (
			$self->parse_nagios_cfg( 'contactgroup',
				$self->parse_nagios_cfg( 'host', $host, 'contact_groups' ),
				'members' ) =~ m/$user(,|$)/
		  );
	}
	else {

		# Nagios Version 1.X is no more supported so everything is allowd
		$rw = 1;
	}
	return $rw;
}

sub read_ngrapher_cfg {
	my $self = shift;
	$self->{ngraphcfg} = undef;
	if ( open( CFG, "<" . $ngraphcfg ) ) {
		while (<CFG>) {
			if ( m/\s*define\s+config\s*{/ .. /\s*}/ ) {
				s/^\s*//;
				chomp();
				if (m/\s*define\s+config\s*{/) {
					# do nothing;
				} elsif (m/(\w*[_]*\w*)\s+(.*)$/) {
					my $key   = $1;
					my $value = $2;

					# remove trailing blanks...
					$value =~ s/\s*$//;

					# value inside single quotes?
					if ( $value =~ m/^'(.*)'$/ ) {
						$value = $1;

						# value inside double quotes?
					}
					elsif ( $value =~ m/^"(.*)"$/ ) {
						$value = $1;

					}

					$value .= "/" if ( $key =~ m/path$/ && $value !~ m/\/$/ );

					# $ngraphcfg{$key}=$value;
					$self->{ngraphcfg}->{$key} = $value;

				}
			}
		}

		# $loaded_cfg=1;
		$self->{loaded_ngraphcfg} = 1;
		close(CFG);
	}
	return;
}

sub get_ngraphercfg_value {
	my $self = shift;
	my ($key) = @_;

	$self->read_ngrapher_cfg() if ( !$self->{loaded_ngraphcfg} );

	my $rw = undef;
	if ( $key eq 'ngraphcfg' ) {
		$rw = $self->{ngraphcfg};
	}
	elsif ( exists( $self->{ngraphcfg}->{$key} ) ) {
		$rw = $self->{ngraphcfg}->{$key};
	}

	return $rw;
}

sub log_ngraphercfg {
	my $self = shift;
	my $log;

	$self->read_ngrapher_cfg() if ( !$self->{loaded_ngraphcfg} );

	foreach my $key ( sort keys %{ $self->{ngraphcfg} } ) {
		$self->print_log( "CFG: " . $key . " => '" . $self->{ngraphcfg}->{$key}."'")
		  if ( $key ne "" && $self->{log_level} & $LOG_CONFIG );
	}
}

sub get_ngrapher_files {
	my $self = shift;

	my %args = (
		dir   => undef,
		files => undef,
		@_
	);

	my $dir = $self->check_slash( $args{dir} );

	my @files = glob( $dir . '*' );

	foreach (@files) {

		if ( -d $_ ) {
			$self->get_ngrapher_files( dir => $_, files => $args{files} );
		}
		elsif ( defined($_) && $_ =~ m/\.ncfg$/ && -e $_ ) {
			push @{ $args{files} }, $_;
		}

	}
}

sub check_slash {
	my $self = shift;

	my ($dir) = @_;
	$dir .= '/' unless ( $dir =~ m/\/$/i );

	return $dir;
}

sub read_ngrapher_def {
	my $self = shift;
	my ($file) = @_;

	$file = $ngraphcfg unless ( defined($file) );
	return 0 unless ( -e $file );

	my ( %tmp, $type, @files );
	if ( open( CFG, "<" . $file ) ) {
		while (<CFG>) {

			# Remove comments
			s/\s*#.*?$//;

			if (m/(cfg_file|cfg_dir)\s*=\s*(.*?)$/i) {
				my $res = $2;
				if ( $1 =~ m/cfg_file/i ) {
					push @files, $res;
				}
				elsif ( $1 =~ m/cfg_dir/ ) {
					my @tmp_files;
					$self->get_ngrapher_files(
						dir   => $res,
						files => \@tmp_files
					);
					push @files, @tmp_files if ( @tmp_files > 0 );
				}
			}

			elsif ( m/\s*define\s+(ngraph|nmgraph)\s*{/ .. /\s*}/ ) {
				s/^\s*//;
				chomp();

				# new block? => initilize
				if (m/\s*define\s+(ngraph|nmgraph)\s*{/) {
					%tmp = ();

					$self->print_log("CFG: ==> $1 definition: ")
					  if ( $self->{log_level} & $LOG_CONFIG );
					$tmp{define} = $1;

					# block body? => fetch values
				}
				elsif (m/([a-z_]+)\s+(.*)$/i) {

					my $key   = $1;
					my $value = $2;

					# value inside single quotes?
					if ( $value =~ m/^'(.*)'$/ ) {
						$value = $1;

						# value inside double quotes?
					}
					elsif ( $value =~ m/^"(.*)"$/ ) {
						$value = $1;

						# or remove all trailing blanks
					}
					else {
						chomp($value);
						$value =~ s/\s*$//;
					}

					$tmp{$key} = $value;

					# end of block? => save values
				}
				elsif (m/}/) {
					if ( ( keys(%tmp) ) > 0 ) {
						push @{ $self->{ngraphdef} }, {%tmp}
						  if ( $tmp{define} =~ m/^ngraph/i );
						push @{ $self->{nmgraphdef} }, {%tmp}
						  if ( $tmp{define} =~ m/^nmgraph/i );

						#dump config into logfile
						for my $key ( sort keys %tmp ) {
							$self->print_log(
								"CFG: " . $key . " => '" . $tmp{$key} . "'" )
							  if ( $self->{log_level} & $LOG_CONFIG );
						}
						%tmp = ();
					}
				}
			}
		}

		$self->{loaded_ngraphdef} = 1;
		close(CFG);
	}

	foreach (@files) { $self->read_ngrapher_def($_); }

	return;
}

sub get_ngrapherdef {
	my $self = shift;
	my @out  = ();
	$self->read_ngrapher_def() if ( !$self->{loaded_ngraphdef} );
	my ( $service, $type ) = @_;

	if ( defined $self->{ngraphdef} ) {
		@out = @{ $self->{ngraphdef} };
	}

	@out = grep $service =~ m/$_->{service_name}/i, @out
	  if ( defined($service) );

	if ( defined($type) && $type =~ m/graph/i ) {
		@out = grep !exists $_->{type}, @out;
	}
	elsif ( defined($type) ) {
		@out = grep $_->{type} eq $type, @out;
	}

	return (@out);
}

sub get_nmgrapherdef {
	my $self = shift;

	my @out = ();
	my ($name) = @_;
	$self->read_ngrapher_def() if ( !$self->{loaded_ngraphdef} );

	if ( defined $self->{nmgraphdef} ) {
		push @out, @{ $self->{nmgraphdef} };
	}

	if ( defined($name) && scalar @out > 0 ) {
		@out = grep $name =~ m/^$_->{service_name}/i, @out;
		if ( @out > 0 ) {
			return %{ $out[0] };
		}
		else {
			return 0;
		}
	}

	return (@out);
}

sub read_ngrapher_xml {
	my $self     = shift;
	my $rrd_path =
	  $self->check_slash( $self->get_ngraphercfg_value('rrdpath') );
	my $file      = $rrd_path . $xmlfile;
	my $indexfile = $file;

	$indexfile =~ s/xml/ngraph/;

	# new style because of performance / GM 2005-07-15
	if ( -e $indexfile ) {
		my $dump = '';
		open( IF, "<$indexfile" );
		while (<IF>) {
			$dump .= $_;
		}

		# make data untainted ...
		$dump =~ s/\$VAR1 = //;
		$self->{rrdhash}        = \%{ eval($dump) };
		$self->{loaded_rrdhash} = 1;

	}

	return;
}

sub get_ngrapherxml {
	my $self = shift;

	$self->read_ngrapher_xml() unless ( $self->{loaded_rrdhash} );
	return ( $self->{rrdhash} );
}

sub write_ngrapherxml {
	my $self = shift;

	my ( $filename, $filename2, $hash ) = @_;
	my $indexfile = $filename2;
	$indexfile =~ s/xml/ngraph/;

	#	unlink ($filename) if (-e $filename);
	#	my $xs = new XML::Simple(XMLDecl => 1,
	#				 RootName => 'main',
	#				 KeyAttr => 'name',
	#				 ForceArray => 1);
	#
	#	open (XF, ">$filename");
	#	print XF $xs->XMLout($hash);
	#	close (XF);
	#
	#	unlink ($filename2) if (-e $filename2);
	#
	#	move ( $filename, $filename2 );

	# new style because of performance / GM 2005-07-15
	unlink($indexfile) if ( -e $indexfile );
	open( IF, ">$indexfile" );
	print IF Dumper($hash);
	close(IF);

	return;
}

sub get_random_letters {
	my $self = shift;

	my ($anzahl) = @_;
	$anzahl = 9 unless ( defined $anzahl );
	$anzahl--;
	my ($string);
	$string = join '', ( "A" .. "Z", 0 .. 9 )[ map { rand 36 } 0 .. $anzahl ];
	return $string;
}

sub get_next_color {
	my $self = shift;

	$self->{colors_count} = 0
	  if ( $self->{colors_count} >= @{ $self->{colors} } );
	my $out = '#' . $self->{colors}->[ $self->{colors_count} ];
	$self->{colors_count}++;
	return $out;
}

sub set_colors {
	my $self = shift;

	my ($new_colors) = @_;
	$new_colors =~ s/\s+//gi;

	my @new_colors_arr = split( /,/i, $new_colors );

	$self->{colors} = \@new_colors_arr;
}

sub time_need {
	my $self = shift;
	if ($NagiosGrapher::DEBUG) {
		my %args = (
			type => undef,
			msg  => undef,
			flag => undef,
			@_
		);
		if ( $args{type} eq 'start' ) {
			$self->{time_start} = [gettimeofday];
			return;
		}
		elsif ( $args{type} eq 'stop' && $args{msg} ) {
			$self->{time_stop} = [gettimeofday];
			my $elapsed =
			  tv_interval( $self->{time_start}, $self->{time_stop} );
			carp 'Benchmark, ' . $args{msg} . ': ' . $elapsed . ' seconds';

		}
	}
}

sub fill_text {
	my $self = shift;

	my ( $text, $length ) = @_;
	return if ( length($text) <= 0 );
	$length = 0 if ( !defined($length) );
	my $add = $length - length($text);
	return $text if ( $add <= 0 );

	for ( my $i = 0 ; $i < $add ; $i++ ) {
		$text .= ' ';
	}

	return $text;
}

sub format_eol {
	my $self = shift;
	my ($eol);
	my ($format) = @_;
	my ( $name, $fill ) = split( /:/, $format );

	if ( $name =~ m/justify/i ) {
		$eol = '\j';
	}
	elsif ( $name =~ m/left/i ) {
		$eol = '\l';
	}
	elsif ( $name =~ m/right/i ) {
		$eol = '\r';
	}
	elsif ( $name =~ m/center/i ) {
		$eol = '\c';
	}
	elsif ( $name =~ m/nospace/i ) {
		$eol = '\g';
	}
	else {
		$eol = '';
	}

	if ( defined($fill) && $fill > 0 ) {
		$eol = NagiosGrapher::fill_text( '', $fill ) . $eol;
	}

	return $eol;
}

sub rrd_info {
	my $self = shift;
	my ( $host, $rrdname ) = @_;
	my $filename =
	    $self->check_slash( $self->get_ngraphercfg_value('rrdpath') ) . $host
	  . '/'
	  . $rrdname;
	if ( -e $filename ) {
		return RRDs::info($filename);
	}
}

sub graph_image {
	my $self = shift;

	my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday ) =
	  localtime( time() );
	my $date = sprintf(
		"[%02d\\:%02d/%04d-%02d-%02d]",
		$hour, $min, $year + 1900,
		$mon + 1, $mday
	);

	my %args = (
		'imageformat' => 'PNG',
		'options'     => undef,
		'height'      => undef,
		'width'       => undef,
		'start'       => undef,
		'end'		  => undef,
		'title'       => undef,
		'vertical'    => undef,
		'values'      => undef,
		'graphs'      => undef,
		'prints'      => undef,
		'comment'     => undef,
		@_
	);

	# seperates the times so no error is thrown
	my (@times);
	push @times, $args{'start'} if ($args{'start'});
	push @times, $args{'end'} if ($args{'end'});

	RRDs::graph(
		'-',
		'-a' . $args{'imageformat'},
		@{ $args{'options'} },
		'-h' . $args{'height'},
		'-w' . $args{'width'},
		@times,
		'--title=' . $args{'title'},
		'-v ' . $args{'vertical'},
		@{ $args{'values'} },
		@{ $args{'graphs'} },
		@{ $args{'prints'} },
		'COMMENT:\j',
		'COMMENT:\j',
		'COMMENT:'
		  . ( $args{'comment'} ? $args{'comment'} : '' ) . ' '
		  . $date . '\r'
	);

	return RRDs::error;
}

sub error_image {
	my $self = shift;
	my %args = (
		'message' => undef,
		'width'   => 530,
		@_
	);

	# my ($msg) = @_;
	my @lines = split( /\n/, $args{'message'} );

	my $lheight = scalar(@lines) * 12.5 + 4;
	my $im      = new GD::Image( $args{'width'}, $lheight );

	# allocate some colors
	my $white = $im->colorAllocate( 255, 255, 255 );
	my $black = $im->colorAllocate( 0,   0,   0 );
	my $red   = $im->colorAllocate( 255, 0,   0 );

	$im->rectangle( 1, 1, $args{'width'} - 1, $lheight - 1, $red );

	# make the background transparent and interlaced
	$im->transparent($white);
	$im->interlaced('true');
	my $line = 2;
	foreach (@lines) {
		$im->string( gdSmallFont, 10, $line, $_, $red );
		$line += 12;
	}

	binmode STDOUT;
	print '[RETURN:1]' . $im->png;
}

sub save_object {
	my $self = shift;

	my $user  = getpwnam( $self->get_ngraphercfg_value('user') );
	my $group = getgrnam( $self->get_ngraphercfg_value('group') );

	if ( -e $NagiosGrapher::object_cache ) {
		unlink($NagiosGrapher::object_cache);
	}

	if (   store( $self, $NagiosGrapher::object_cache )
		&& $user > 0
		&& $group > 0 )
	{
		chown $user, $group, $NagiosGrapher::object_cache;
		return 1;
	}

	return;

}

sub getInstance {
	if ( !defined($uniqueInstance) ) {
		if ( -e $NagiosGrapher::object_cache ) {
			$uniqueInstance = retrieve $NagiosGrapher::object_cache;
		}

		# Generate an error if the cache could not be loaded
		#if (!defined($uniqueInstance)) {
		#	$uniqueInstance = new NagiosGrapher();
		#}
	}

	return $uniqueInstance;
}

sub daemon_write_pidfile {
	my $self = shift;
	my $file = $self->get_ngraphercfg_value('pidfile');
	unlink($file) if ( -e $file );
	if ( open( PIDFILE, '>' . $file ) ) {
		print PIDFILE "$$\n";
		close(PIDFILE);
		return 1;
	}
	else {
		die("Can not write pidfile '$file'");
	}
	return;
}

sub daemon_change_user {
	my $self = shift;

	my $user  = getpwnam( $self->get_ngraphercfg_value('user') );
	my $group = getgrnam( $self->get_ngraphercfg_value('group') );

	if ( $user > 0 && $group > 0 ) {

		my $pidfile = $self->get_ngraphercfg_value('pidfile');

		if ( -e $pidfile ) {
			chown $user, $group, $pidfile;
		}

		POSIX::setuid($user);
		POSIX::setgid($group);

		return 1;
	}

	return;
}

sub daemon_prepare {
	my $self  = shift;
	my $user  = getpwnam( $self->get_ngraphercfg_value('user') );
	my $group = getgrnam( $self->get_ngraphercfg_value('group') );

	if ( $user > 0 && $group > 0 ) {
		my $dir = dirname( $self->get_ngraphercfg_value('pidfile') );
		mkdir( $dir, 0777 );
		chown $user, $group, $dir;
	}
}

sub daemon_cleanup {
	my $self    = shift;
	my $pidfile = $self->get_ngraphercfg_value('pidfile');
	unlink $pidfile;
	unlink $NagiosGrapher::object_cache;

	# rmdir (dirname($pidfile));
	return 1;
}


1;
